/*
 * Callback functions
 * Copyright (C) 2001 Jan Oberlnder <mindriot@gmx.net>
 *
 * This file is part of netload_applet.
 *
 * netload_applet is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: callbacks.c,v 1.1.1.1 2001/03/08 03:01:56 mindriot Exp $
 */

#include <config.h>
#include "callbacks.h"

void
on_load_theme_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *msgbox;

	DEBUG_MESSAGE ("on_load_theme_button_clicked.\n");

	msgbox = gnome_message_box_new (_("Sorry, this is not implemented yet!"), GNOME_MESSAGE_BOX_INFO, _("OK"), NULL);
	gnome_dialog_run (GNOME_DIALOG (msgbox));
}

void
on_save_theme_button_clicked (GtkButton *button, gpointer user_data)
{
	DEBUG_MESSAGE ("on_save_theme_button_clicked.\n");
}

void
on_ok_button_clicked (GtkButton *button, gpointer user_data)
{
	DEBUG_MESSAGE ("on_ok_button_clicked.\n");

	update_scheme ();
	pref_window_visible = FALSE;
	gtk_widget_destroy (pref_dialog);
}

void
on_close_button_clicked (GtkButton *button, gpointer user_data)
{
	DEBUG_MESSAGE ("on_close_button_clicked.\n");

	pref_window_visible = FALSE;
	gtk_widget_destroy (pref_dialog);
}

void
on_apply_button_clicked (GtkButton *button, gpointer user_data)
{
	DEBUG_MESSAGE ("on_apply_button_clicked.\n");

	update_scheme ();
}

gboolean
on_pref_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	DEBUG_MESSAGE ("on_pref_window_delete_event.\n");
	pref_window_visible = FALSE;
	return FALSE;
}

gboolean
on_pref_window_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	DEBUG_MESSAGE ("on_pref_window_destroy_event.\n");
	pref_window_visible = FALSE;
	return TRUE;
}

void
pref_handler(AppletWidget *applet, gpointer data)
{
	if (!pref_window_visible)
		pref_dialog = create_pref_window();
	pref_window_visible = TRUE;
	gtk_widget_show (pref_dialog);
}

void
destroy_handler_main (GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

void
about_handler (AppletWidget *applet, gpointer data)
{
	const gchar *authors[] = {
		"Jan Oberlnder <mindriot@gmx.net>",
		NULL
	};

	gtk_widget_show(gnome_about_new(_("Network Load"), VERSION,
					"(c) 2000 by Jan Oberlnder",
					(const gchar **) authors,
					_("A simple applet displaying the network transmit and receive rates "
					"of a particular network device."),
					"netload_applet.png"));
}

gboolean
redraw_handler (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	gdk_gc_set_clip_rectangle(gc, &event->area);
	draw_values();
	gdk_gc_set_clip_rectangle(gc, NULL);
	return TRUE;	
}

gint
timeout_handler (gpointer data)
{
	shift_values();
	get_byte_counts();
	scale_values();
	draw_values();
	return TRUE;
}

gboolean
session_save_handler (GtkWidget *w, const char *privcfgpath, const char *globcfgpath)
{
#if (DEBUG == 1)
	printf ("session_save-handler: writing config to %s.\n", privcfgpath);
#endif

	gnome_config_push_prefix (privcfgpath);

	/*
	 * OK, now we can write our configuration.
	 */

	gnome_config_set_string ("main/rx_load_font_name", display_scheme.rx_load_font_name);
	gnome_config_set_string ("main/tx_load_font_name", display_scheme.tx_load_font_name);
	gnome_config_set_string ("main/max_load_font_name", display_scheme.max_load_font_name);
	gnome_config_set_string ("main/dev_font_name", display_scheme.dev_font_name);

	gnome_config_set_int ("main/background_color_red", display_scheme.background_color.red);
	gnome_config_set_int ("main/background_color_green", display_scheme.background_color.green);
	gnome_config_set_int ("main/background_color_blue", display_scheme.background_color.blue);
	gnome_config_set_int ("main/rx_graph_color_red", display_scheme.rx_graph_color.red);
	gnome_config_set_int ("main/rx_graph_color_green", display_scheme.rx_graph_color.green);
	gnome_config_set_int ("main/rx_graph_color_blue", display_scheme.rx_graph_color.blue);
	gnome_config_set_int ("main/tx_graph_color_red", display_scheme.tx_graph_color.red);
	gnome_config_set_int ("main/tx_graph_color_green", display_scheme.tx_graph_color.green);
	gnome_config_set_int ("main/tx_graph_color_blue", display_scheme.tx_graph_color.blue);
	gnome_config_set_int ("main/rx_avg_color_red", display_scheme.rx_avg_color.red);
	gnome_config_set_int ("main/rx_avg_color_green", display_scheme.rx_avg_color.green);
	gnome_config_set_int ("main/rx_avg_color_blue", display_scheme.rx_avg_color.blue);
	gnome_config_set_int ("main/tx_avg_color_red", display_scheme.tx_avg_color.red);
	gnome_config_set_int ("main/tx_avg_color_green", display_scheme.tx_avg_color.green);
	gnome_config_set_int ("main/tx_avg_color_blue", display_scheme.tx_avg_color.blue);
	gnome_config_set_int ("main/rx_load_color_red", display_scheme.rx_load_color.red);
	gnome_config_set_int ("main/rx_load_color_green", display_scheme.rx_load_color.green);
	gnome_config_set_int ("main/rx_load_color_blue", display_scheme.rx_load_color.blue);
	gnome_config_set_int ("main/tx_load_color_red", display_scheme.tx_load_color.red);
	gnome_config_set_int ("main/tx_load_color_green", display_scheme.tx_load_color.green);
	gnome_config_set_int ("main/tx_load_color_blue", display_scheme.tx_load_color.blue);
	gnome_config_set_int ("main/max_load_color_red", display_scheme.max_load_color.red);
	gnome_config_set_int ("main/max_load_color_green", display_scheme.max_load_color.green);
	gnome_config_set_int ("main/max_load_color_blue", display_scheme.max_load_color.blue);
	gnome_config_set_int ("main/dev_color_red", display_scheme.dev_color.red);
	gnome_config_set_int ("main/dev_color_green", display_scheme.dev_color.green);
	gnome_config_set_int ("main/dev_color_blue", display_scheme.dev_color.blue);
	gnome_config_set_int ("main/top_edge_color_red", display_scheme.top_edge_color.red);
	gnome_config_set_int ("main/top_edge_color_green", display_scheme.top_edge_color.green);
	gnome_config_set_int ("main/top_edge_color_blue", display_scheme.top_edge_color.blue);
	gnome_config_set_int ("main/bottom_edge_color_red", display_scheme.bottom_edge_color.red);
	gnome_config_set_int ("main/bottom_edge_color_green", display_scheme.bottom_edge_color.green);
	gnome_config_set_int ("main/bottom_edge_color_blue", display_scheme.bottom_edge_color.blue);

	gnome_config_set_int ("main/rx_load_position", display_scheme.rx_load_position);
	gnome_config_set_int ("main/tx_load_position", display_scheme.tx_load_position);
	gnome_config_set_int ("main/max_load_position", display_scheme.max_load_position);
	gnome_config_set_int ("main/dev_position", display_scheme.dev_position);

	gnome_config_set_bool ("main/show_rx_graph", display_scheme.show_rx_graph);
	gnome_config_set_bool ("main/show_tx_graph", display_scheme.show_tx_graph);
	gnome_config_set_bool ("main/show_rx_avg", display_scheme.show_rx_avg);
	gnome_config_set_bool ("main/show_tx_avg", display_scheme.show_tx_avg);
	gnome_config_set_bool ("main/show_rx_load", display_scheme.show_rx_load);
	gnome_config_set_bool ("main/show_tx_load", display_scheme.show_tx_load);
	gnome_config_set_bool ("main/show_max_load", display_scheme.show_max_load);
	gnome_config_set_bool ("main/show_dev", display_scheme.show_dev);

	gnome_config_set_int ("main/rx_graph_style", display_scheme.rx_graph_style);
	gnome_config_set_int ("main/tx_graph_style", display_scheme.tx_graph_style);
	gnome_config_set_int ("main/graph_order", display_scheme.graph_order);

	gnome_config_set_int ("main/width", display_scheme.width);
	gnome_config_set_int ("main/height", display_scheme.height);
	gnome_config_set_bool ("main/fixed_width", display_scheme.fixed_width);
	gnome_config_set_bool ("main/fixed_height", display_scheme.fixed_height);

	gnome_config_set_string ("main/device", display_scheme.device);
	gnome_config_set_int ("main/timeout", display_scheme.timeout);

	gnome_config_pop_prefix();
	gnome_config_sync();
	gnome_config_drop_all();

	return FALSE;
}

#ifdef HAVE_PANEL_PIXEL_SIZE
void
change_pixel_size_handler (GtkWidget *widget, int size, gpointer data)
{
#if (DEBUG == 1)
        printf("change_pixel_size_handler: size changed to %d pixels.\n", size);
#endif
	/*
	 * adapt all data to new panel size and store the panel size in the panel_size global variable
	 */
	panel_size = size;
	if (!display_scheme.fixed_width)
		display_scheme.width = size;
	if (!display_scheme.fixed_height)
		display_scheme.height = size;

	if (!(display_scheme.fixed_width && display_scheme.fixed_height)) {
		resize_count_fields();
		gtk_drawing_area_size(GTK_DRAWING_AREA (draw), display_scheme.width, display_scheme.height);
	}

#if (DEBUG == 1)
	printf("change_pixel_size_handler: new applet size is %dx%d pixels.\n", display_scheme.width, display_scheme.height);
#endif
}

#endif
