/*
 * Interface creation functions
 * Copyright (C) 2001 Jan Oberlnder <mindriot@gmx.net>
 *
 * This file is part of netload_applet.
 *
 * netload_applet is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: interface.c,v 1.3 2001/10/06 11:45:22 mindriot Exp $
 */

#include <config.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "interface.h"

/*
 * Updates the display scheme from the dialog settings.
 */
void
update_scheme (void)
{
	DisplayScheme new_scheme;

	/*
	 * get settings and store into our private scheme
	 */
	new_scheme.fixed_width = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (fixed_width));
	new_scheme.fixed_height = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (fixed_height));
	new_scheme.width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (width_spinbutton));
	new_scheme.height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (height_spinbutton));
	new_scheme.timeout = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (timeout_spinbutton));
	new_scheme.device = g_strdup (device_list[gtk_option_menu_get_history (GTK_OPTION_MENU (device_selector))]);
       	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (background_color_picker), &new_scheme.background_color.red,
				    &new_scheme.background_color.green, &new_scheme.background_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (top_edge_color_picker), &new_scheme.top_edge_color.red,
				    &new_scheme.top_edge_color.green, &new_scheme.top_edge_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (bottom_edge_color_picker), &new_scheme.bottom_edge_color.red,
				    &new_scheme.bottom_edge_color.green, &new_scheme.bottom_edge_color.blue, NULL);
	new_scheme.show_rx_graph = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_rx_graph_button));
	new_scheme.show_tx_graph = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_tx_graph_button));
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rx_on_top_button)))
		new_scheme.graph_order = RX_ON_TOP;
	else
		new_scheme.graph_order = TX_ON_TOP;

	new_scheme.show_rx_avg = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_rx_avg_load_button));
	new_scheme.show_tx_avg = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_tx_avg_load_button));
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (rx_graph_color_picker), &new_scheme.rx_graph_color.red,
				    &new_scheme.rx_graph_color.green, &new_scheme.rx_graph_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (tx_graph_color_picker), &new_scheme.tx_graph_color.red,
				    &new_scheme.tx_graph_color.green, &new_scheme.tx_graph_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (rx_avg_load_color_picker), &new_scheme.rx_avg_color.red,
				    &new_scheme.rx_avg_color.green, &new_scheme.rx_avg_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (tx_avg_load_color_picker), &new_scheme.tx_avg_color.red,
				    &new_scheme.tx_avg_color.green, &new_scheme.tx_avg_color.blue, NULL);
	new_scheme.rx_graph_style = gtk_option_menu_get_history (GTK_OPTION_MENU (rx_graph_style_selector));
	new_scheme.tx_graph_style = gtk_option_menu_get_history (GTK_OPTION_MENU (tx_graph_style_selector));

	new_scheme.show_rx_load = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rx_rate_button));
	new_scheme.show_tx_load = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tx_rate_button));
	new_scheme.show_max_load = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (max_rate_button));
	new_scheme.show_dev = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (device_name_button));

	new_scheme.rx_load_position = gtk_option_menu_get_history (GTK_OPTION_MENU (rx_vpos_selector)) << 2;
	new_scheme.tx_load_position = gtk_option_menu_get_history (GTK_OPTION_MENU (tx_vpos_selector)) << 2;
	new_scheme.max_load_position = gtk_option_menu_get_history (GTK_OPTION_MENU (max_vpos_selector)) << 2;
	new_scheme.dev_position = gtk_option_menu_get_history (GTK_OPTION_MENU (device_vpos_selector)) << 2;
	new_scheme.rx_load_position |= gtk_option_menu_get_history (GTK_OPTION_MENU (rx_hpos_selector));
	new_scheme.tx_load_position |= gtk_option_menu_get_history (GTK_OPTION_MENU (tx_hpos_selector));
	new_scheme.max_load_position |= gtk_option_menu_get_history (GTK_OPTION_MENU (max_hpos_selector));
	new_scheme.dev_position |= gtk_option_menu_get_history (GTK_OPTION_MENU (device_hpos_selector));
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (rx_color_picker), &new_scheme.rx_load_color.red,
				    &new_scheme.rx_load_color.green, &new_scheme.rx_load_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (tx_color_picker), &new_scheme.tx_load_color.red,
				    &new_scheme.tx_load_color.green, &new_scheme.tx_load_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (max_color_picker), &new_scheme.max_load_color.red,
				    &new_scheme.max_load_color.green, &new_scheme.max_load_color.blue, NULL);
	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (device_color_picker), &new_scheme.dev_color.red,
				    &new_scheme.dev_color.green, &new_scheme.dev_color.blue, NULL);
	/*
	 * get font settings. If the picker was nerver used, use old font.
	 */
	new_scheme.rx_load_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (rx_font_picker)));
	new_scheme.tx_load_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (tx_font_picker)));
	new_scheme.max_load_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (max_font_picker)));
	new_scheme.dev_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (device_font_picker)));

	/*
	 * Now, first deactivate the timeout so we can update all values
	 */
	gtk_timeout_remove (timeout_id);

	/*
	 * If the device was changed, we need to reinit the byte counts.
	 */
	if (strcmp(display_scheme.device, new_scheme.device)) {
		oldwidth = 0;
	}

	/*
	 * deinit display_scheme and init the new scheme, then copy data
	 */
	free_display_scheme_data ();
	display_scheme = new_scheme;
	init_display_scheme ();

	/*
	 * now update width and height where necessary, and reinit the timeout
	 * get width and height from current size where non-fixed
	 */
#ifdef HAVE_PANEL_PIXEL_SIZE
	if (!display_scheme.fixed_width) display_scheme.width = panel_size;
	if (!display_scheme.fixed_height) display_scheme.height = panel_size;
#endif
	resize_count_fields ();
	gtk_drawing_area_size(GTK_DRAWING_AREA (draw), display_scheme.width, display_scheme.height);
	
	/*
	 * reactivate timeout
	 */
	timeout_id = gtk_timeout_add(display_scheme.timeout, timeout_handler, NULL);
}

GtkWidget*
create_pref_window (void)
{
	guint i, default_value = 0;

	/*
	 * Update list of devices.
	 */
	if (device_list) {
		g_strfreev(device_list);
		device_list = get_available_devices();
	}

	tooltips = gtk_tooltips_new ();

	accel_group = gtk_accel_group_new ();

	pref_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (pref_window), "pref_window", pref_window);
	gtk_container_set_border_width (GTK_CONTAINER (pref_window), 4);
	gtk_window_set_title (GTK_WINDOW (pref_window), _("Preferences"));
	gtk_window_set_position (GTK_WINDOW (pref_window), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (pref_window), FALSE, FALSE, FALSE);

	main_box = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (main_box);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "main_box", main_box,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_box);
	gtk_container_add (GTK_CONTAINER (pref_window), main_box);

	notebook = gtk_notebook_new ();
	gtk_widget_ref (notebook);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "notebook", notebook,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (main_box), notebook, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (notebook), 4);

	nbox1 = gtk_vbox_new (FALSE, 2);
	gtk_widget_ref (nbox1);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nbox1", nbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nbox1);
	gtk_container_add (GTK_CONTAINER (notebook), nbox1);
	gtk_container_set_border_width (GTK_CONTAINER (nbox1), 4);

	info1 = gtk_label_new (_("Set the applet size and the device to monitor."));
	gtk_widget_ref (info1);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "info1", info1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (info1);
	gtk_box_pack_start (GTK_BOX (nbox1), info1, FALSE, FALSE, 0);

	nseparator1 = gtk_hseparator_new ();
	gtk_widget_ref (nseparator1);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nseparator1", nseparator1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nseparator1);
	gtk_box_pack_start (GTK_BOX (nbox1), nseparator1, FALSE, TRUE, 0);

	size_table = gtk_table_new (2, 3, FALSE);
	gtk_widget_ref (size_table);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "size_table", size_table,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (size_table);
	gtk_box_pack_start (GTK_BOX (nbox1), size_table, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (size_table), 4);
	gtk_table_set_row_spacings (GTK_TABLE (size_table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (size_table), 4);

	fixed_width = gtk_check_button_new_with_label (_("Use fixed width."));
	gtk_widget_ref (fixed_width);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "fixed_width", fixed_width,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (fixed_width);
	gtk_table_attach (GTK_TABLE (size_table), fixed_width, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	fixed_height = gtk_check_button_new_with_label (_("Use fixed height."));
	gtk_widget_ref (fixed_height);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "fixed_height", fixed_height,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (fixed_height);
	gtk_table_attach (GTK_TABLE (size_table), fixed_height, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	width_label = gtk_label_new (_("Width in pixels:"));
	gtk_widget_ref (width_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "width_label", width_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (width_label);
	gtk_table_attach (GTK_TABLE (size_table), width_label, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (width_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (width_label), 1, 0.5);

	height_label = gtk_label_new (_("Height in pixels:"));
	gtk_widget_ref (height_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "height_label", height_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (height_label);
	gtk_table_attach (GTK_TABLE (size_table), height_label, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (height_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (height_label), 1, 0.5);

	width_spinbutton_adj = gtk_adjustment_new (48, 16, 512, 1, 10, 10);
	width_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (width_spinbutton_adj), 1, 0);
	gtk_widget_ref (width_spinbutton);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "width_spinbutton", width_spinbutton,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (width_spinbutton);
	gtk_table_attach (GTK_TABLE (size_table), width_spinbutton, 2, 3, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	height_spinbutton_adj = gtk_adjustment_new (48, 1, 512, 1, 10, 10);
	height_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (height_spinbutton_adj), 1, 0);
	gtk_widget_ref (height_spinbutton);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "height_spinbutton", height_spinbutton,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (height_spinbutton);
	gtk_table_attach (GTK_TABLE (size_table), height_spinbutton, 2, 3, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	panel_table = gtk_table_new (5, 2, FALSE);
	gtk_widget_ref (panel_table);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "panel_table", panel_table,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (panel_table);
	gtk_box_pack_start (GTK_BOX (nbox1), panel_table, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (panel_table), 4);
	gtk_table_set_row_spacings (GTK_TABLE (panel_table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (panel_table), 4);

	background_label = gtk_label_new (_("Background color:"));
	gtk_widget_ref (background_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "background_label", background_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (background_label);
	gtk_table_attach (GTK_TABLE (panel_table), background_label, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (background_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (background_label), 0, 0.5);

	topleft_label = gtk_label_new (_("Top-left edge color:"));
	gtk_widget_ref (topleft_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "topleft_label", topleft_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (topleft_label);
	gtk_table_attach (GTK_TABLE (panel_table), topleft_label, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (topleft_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (topleft_label), 0, 0.5);

	bottomright_label = gtk_label_new (_("Bottom-right edge color:"));
	gtk_widget_ref (bottomright_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "bottomright_label", bottomright_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (bottomright_label);
	gtk_table_attach (GTK_TABLE (panel_table), bottomright_label, 0, 1, 4, 5,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (bottomright_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (bottomright_label), 0, 0.5);

	monitor_label = gtk_label_new (_("Monitor device:"));
	gtk_widget_ref (monitor_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "monitor_label", monitor_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (monitor_label);
	gtk_table_attach (GTK_TABLE (panel_table), monitor_label, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (monitor_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (monitor_label), 0, 0.5);

	refresh_label = gtk_label_new (_("Refresh timeout (ms):"));
	gtk_widget_ref (refresh_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "refresh_label", refresh_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (refresh_label);
	gtk_table_attach (GTK_TABLE (panel_table), refresh_label, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (refresh_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (refresh_label), 0, 0.5);

	background_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (background_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "background_color_picker", background_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (background_color_picker);
	gtk_table_attach (GTK_TABLE (panel_table), background_color_picker, 1, 2, 2, 3,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (background_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (background_color_picker), FALSE);

	top_edge_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (top_edge_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "top_edge_color_picker", top_edge_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (top_edge_color_picker);
	gtk_table_attach (GTK_TABLE (panel_table), top_edge_color_picker, 1, 2, 3, 4,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (top_edge_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (top_edge_color_picker), FALSE);

	bottom_edge_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (bottom_edge_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "bottom_edge_color_picker", bottom_edge_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (bottom_edge_color_picker);
	gtk_table_attach (GTK_TABLE (panel_table), bottom_edge_color_picker, 1, 2, 4, 5,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (bottom_edge_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (bottom_edge_color_picker), FALSE);

	timeout_spinbutton_adj = gtk_adjustment_new (1000, 200, 10000, 1, 10, 10);
	timeout_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (timeout_spinbutton_adj), 1, 0);
	gtk_widget_ref (timeout_spinbutton);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "timeout_spinbutton", timeout_spinbutton,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (timeout_spinbutton);
	gtk_table_attach (GTK_TABLE (panel_table), timeout_spinbutton, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (timeout_spinbutton), TRUE);

	device_selector = gtk_option_menu_new ();
	gtk_widget_ref (device_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_selector", device_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_selector);
	gtk_table_attach (GTK_TABLE (panel_table), device_selector, 1, 2, 0, 1,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (device_selector), 4);
	device_selector_menu = gtk_menu_new ();

	default_value = i = 0;
	while (device_list[i]) {
		glade_menuitem = gtk_menu_item_new_with_label (g_strdup (device_list[i]));
		gtk_widget_show (glade_menuitem);
		gtk_menu_append (GTK_MENU (device_selector_menu), glade_menuitem);

		if (!strcasecmp (device_list[i], display_scheme.device)) {
			default_value = i;
#if (DEBUG == 1)
			printf ("create_pref_window: default device = %i ('%s').\n", i, device_list[i]);
#endif
		}
		i++;
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (device_selector), device_selector_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (device_selector), default_value);

	nlabel1 = gtk_label_new (_("Panel"));
	gtk_widget_ref (nlabel1);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nlabel1", nlabel1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nlabel1);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), nlabel1);

	nbox2 = gtk_vbox_new (FALSE, 4);
	gtk_widget_ref (nbox2);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nbox2", nbox2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nbox2);
	gtk_container_add (GTK_CONTAINER (notebook), nbox2);
	gtk_container_set_border_width (GTK_CONTAINER (nbox2), 4);

	info2 = gtk_label_new (_("Decide which graphs to show, and pick their style and color."));
	gtk_widget_ref (info2);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "info2", info2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (info2);
	gtk_box_pack_start (GTK_BOX (nbox2), info2, FALSE, FALSE, 0);

	nseparator2 = gtk_hseparator_new ();
	gtk_widget_ref (nseparator2);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nseparator2", nseparator2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nseparator2);
	gtk_box_pack_start (GTK_BOX (nbox2), nseparator2, FALSE, TRUE, 0);

	graph_hbox = gtk_hbox_new (FALSE, 4);
	gtk_widget_ref (graph_hbox);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "graph_hbox", graph_hbox,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (graph_hbox);
	gtk_box_pack_start (GTK_BOX (nbox2), graph_hbox, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (graph_hbox), 4);

	rx_frame = gtk_frame_new (_("RX graph"));
	gtk_widget_ref (rx_frame);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_frame", rx_frame,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_frame);
	gtk_box_pack_start (GTK_BOX (graph_hbox), rx_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_frame), 4);

	rx_vbox = gtk_vbox_new (FALSE, 4);
	gtk_widget_ref (rx_vbox);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_vbox", rx_vbox,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_vbox);
	gtk_container_add (GTK_CONTAINER (rx_frame), rx_vbox);

	show_rx_graph_button = gtk_check_button_new_with_label (_("Display"));
	gtk_widget_ref (show_rx_graph_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "show_rx_graph_button", show_rx_graph_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (show_rx_graph_button);
	gtk_box_pack_start (GTK_BOX (rx_vbox), show_rx_graph_button, FALSE, FALSE, 0);

	/*
	 * NOTE: top_group MUST be set to NULL, otherwise netload crashes
	 * when the pref window is opened a second time!
	 */
	top_group = NULL;

	rx_on_top_button = gtk_radio_button_new_with_label (top_group, _("On top"));
	top_group = gtk_radio_button_group (GTK_RADIO_BUTTON (rx_on_top_button));
	gtk_widget_ref (rx_on_top_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_on_top_button", rx_on_top_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_on_top_button);
	gtk_box_pack_start (GTK_BOX (rx_vbox), rx_on_top_button, FALSE, FALSE, 0);

	show_rx_avg_load_button = gtk_check_button_new_with_label (_("Display average load"));
	gtk_widget_ref (show_rx_avg_load_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "show_rx_avg_load_button", show_rx_avg_load_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (show_rx_avg_load_button);
	gtk_box_pack_start (GTK_BOX (rx_vbox), show_rx_avg_load_button, FALSE, FALSE, 0);

	rx_style_table = gtk_table_new (3, 2, FALSE);
	gtk_widget_ref (rx_style_table);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_style_table", rx_style_table,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_style_table);
	gtk_box_pack_start (GTK_BOX (rx_vbox), rx_style_table, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_style_table), 4);
	gtk_table_set_row_spacings (GTK_TABLE (rx_style_table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (rx_style_table), 4);

	rx_graph_style_selector = gtk_option_menu_new ();
	gtk_widget_ref (rx_graph_style_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_graph_style_selector", rx_graph_style_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_graph_style_selector);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_graph_style_selector, 1, 2, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_graph_style_selector), 4);
	rx_graph_style_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Barchart"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_graph_style_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Linechart"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_graph_style_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (rx_graph_style_selector), rx_graph_style_selector_menu);

	rx_graph_color_label = gtk_label_new (_("Color:"));
	gtk_widget_ref (rx_graph_color_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_graph_color_label", rx_graph_color_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_graph_color_label);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_graph_color_label, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (rx_graph_color_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (rx_graph_color_label), 0, 0.5);

	rx_avg_load_color_label = gtk_label_new (_("Avg. load color:"));
	gtk_widget_ref (rx_avg_load_color_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_avg_load_color_label", rx_avg_load_color_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_avg_load_color_label);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_avg_load_color_label, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (rx_avg_load_color_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (rx_avg_load_color_label), 0, 0.5);

	rx_graph_style_label = gtk_label_new (_("Style:"));
	gtk_widget_ref (rx_graph_style_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_graph_style_label", rx_graph_style_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_graph_style_label);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_graph_style_label, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (rx_graph_style_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (rx_graph_style_label), 0, 0.5);

	rx_graph_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (rx_graph_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_graph_color_picker", rx_graph_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_graph_color_picker);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_graph_color_picker, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_graph_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (rx_graph_color_picker), FALSE);

	rx_avg_load_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (rx_avg_load_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_avg_load_color_picker", rx_avg_load_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_avg_load_color_picker);
	gtk_table_attach (GTK_TABLE (rx_style_table), rx_avg_load_color_picker, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_avg_load_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (rx_avg_load_color_picker), FALSE);

	tx_frame = gtk_frame_new (_("TX graph"));
	gtk_widget_ref (tx_frame);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_frame", tx_frame,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_frame);
	gtk_box_pack_start (GTK_BOX (graph_hbox), tx_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_frame), 4);

	tx_vbox = gtk_vbox_new (FALSE, 4);
	gtk_widget_ref (tx_vbox);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_vbox", tx_vbox,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_vbox);
	gtk_container_add (GTK_CONTAINER (tx_frame), tx_vbox);

	show_tx_graph_button = gtk_check_button_new_with_label (_("Display"));
	gtk_widget_ref (show_tx_graph_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "show_tx_graph_button", show_tx_graph_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (show_tx_graph_button);
	gtk_box_pack_start (GTK_BOX (tx_vbox), show_tx_graph_button, FALSE, FALSE, 0);

	tx_on_top_button = gtk_radio_button_new_with_label (top_group, _("On top"));
	top_group = gtk_radio_button_group (GTK_RADIO_BUTTON (tx_on_top_button));
	gtk_widget_ref (tx_on_top_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_on_top_button", tx_on_top_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_on_top_button);
	gtk_box_pack_start (GTK_BOX (tx_vbox), tx_on_top_button, FALSE, FALSE, 0);

	show_tx_avg_load_button = gtk_check_button_new_with_label (_("Display average load"));
	gtk_widget_ref (show_tx_avg_load_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "show_tx_avg_load_button", show_tx_avg_load_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (show_tx_avg_load_button);
	gtk_box_pack_start (GTK_BOX (tx_vbox), show_tx_avg_load_button, FALSE, FALSE, 0);

	tx_style_table = gtk_table_new (3, 2, FALSE);
	gtk_widget_ref (tx_style_table);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_style_table", tx_style_table,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_style_table);
	gtk_box_pack_start (GTK_BOX (tx_vbox), tx_style_table, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_style_table), 4);
	gtk_table_set_row_spacings (GTK_TABLE (tx_style_table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (tx_style_table), 4);

	tx_graph_style_selector = gtk_option_menu_new ();
	gtk_widget_ref (tx_graph_style_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_graph_style_selector", tx_graph_style_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_graph_style_selector);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_graph_style_selector, 1, 2, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_graph_style_selector), 4);
	tx_graph_style_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Barchart"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_graph_style_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Linechart"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_graph_style_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (tx_graph_style_selector), tx_graph_style_selector_menu);

	tx_graph_color_label = gtk_label_new (_("Color:"));
	gtk_widget_ref (tx_graph_color_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_graph_color_label", tx_graph_color_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_graph_color_label);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_graph_color_label, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (tx_graph_color_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (tx_graph_color_label), 0, 0.5);

	tx_graph_style_label = gtk_label_new (_("Style:"));
	gtk_widget_ref (tx_graph_style_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_graph_style_label", tx_graph_style_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_graph_style_label);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_graph_style_label, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (tx_graph_style_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (tx_graph_style_label), 0, 0.5);

	tx_avg_load_color_label = gtk_label_new (_("Avg. load color:"));
	gtk_widget_ref (tx_avg_load_color_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_avg_load_color_label", tx_avg_load_color_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_avg_load_color_label);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_avg_load_color_label, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (tx_avg_load_color_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (tx_avg_load_color_label), 0, 0.5);

	tx_graph_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (tx_graph_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_graph_color_picker", tx_graph_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_graph_color_picker);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_graph_color_picker, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_graph_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (tx_graph_color_picker), FALSE);

	tx_avg_load_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (tx_avg_load_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_avg_load_color_picker", tx_avg_load_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_avg_load_color_picker);
	gtk_table_attach (GTK_TABLE (tx_style_table), tx_avg_load_color_picker, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_avg_load_color_picker), 4);
	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (tx_avg_load_color_picker), FALSE);

	nlabel2 = gtk_label_new (_("Graph"));
	gtk_widget_ref (nlabel2);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nlabel2", nlabel2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nlabel2);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), nlabel2);

	nbox3 = gtk_vbox_new (FALSE, 4);
	gtk_widget_ref (nbox3);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nbox3", nbox3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nbox3);
	gtk_container_add (GTK_CONTAINER (notebook), nbox3);
	gtk_container_set_border_width (GTK_CONTAINER (nbox3), 4);

	info3 = gtk_label_new (_("Pick the labels to display, and choose their position, font and color."));
	gtk_widget_ref (info3);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "info3", info3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (info3);
	gtk_box_pack_start (GTK_BOX (nbox3), info3, FALSE, FALSE, 0);

	nseparator3 = gtk_hseparator_new ();
	gtk_widget_ref (nseparator3);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nseparator3", nseparator3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nseparator3);
	gtk_box_pack_start (GTK_BOX (nbox3), nseparator3, FALSE, TRUE, 0);

	label_table = gtk_table_new (5, 5, FALSE);
	gtk_widget_ref (label_table);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "label_table", label_table,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_table);
	gtk_box_pack_start (GTK_BOX (nbox3), label_table, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (label_table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (label_table), 4);

	vpos_label = gtk_label_new (_("Vertical position"));
	gtk_widget_ref (vpos_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "vpos_label", vpos_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vpos_label);
	gtk_table_attach (GTK_TABLE (label_table), vpos_label, 1, 2, 0, 1,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);

	hpos_label = gtk_label_new (_("Horizontal position"));
	gtk_widget_ref (hpos_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "hpos_label", hpos_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hpos_label);
	gtk_table_attach (GTK_TABLE (label_table), hpos_label, 2, 3, 0, 1,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);

	color_label = gtk_label_new (_("Color"));
	gtk_widget_ref (color_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "color_label", color_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (color_label);
	gtk_table_attach (GTK_TABLE (label_table), color_label, 3, 4, 0, 1,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);

	font_label = gtk_label_new (_("Font"));
	gtk_widget_ref (font_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "font_label", font_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (font_label);
	gtk_table_attach (GTK_TABLE (label_table), font_label, 4, 5, 0, 1,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);

	tx_rate_button = gtk_check_button_new_with_label (_("TX rate"));
	gtk_widget_ref (tx_rate_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_rate_button", tx_rate_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_rate_button);
	gtk_table_attach (GTK_TABLE (label_table), tx_rate_button, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	rx_rate_button = gtk_check_button_new_with_label (_("RX rate"));
	gtk_widget_ref (rx_rate_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_rate_button", rx_rate_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_rate_button);
	gtk_table_attach (GTK_TABLE (label_table), rx_rate_button, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	max_rate_button = gtk_check_button_new_with_label (_("Max rate"));
	gtk_widget_ref (max_rate_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "max_rate_button", max_rate_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (max_rate_button);
	gtk_table_attach (GTK_TABLE (label_table), max_rate_button, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	device_name_button = gtk_check_button_new_with_label (_("Device name"));
	gtk_widget_ref (device_name_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_name_button", device_name_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_name_button);
	gtk_table_attach (GTK_TABLE (label_table), device_name_button, 0, 1, 4, 5,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	rx_vpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (rx_vpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_vpos_selector", rx_vpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_vpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), rx_vpos_selector, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_vpos_selector), 4);
	rx_vpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Top"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Middle"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Bottom"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_vpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (rx_vpos_selector), rx_vpos_selector_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (rx_vpos_selector), 1);

	tx_vpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (tx_vpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_vpos_selector", tx_vpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_vpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), tx_vpos_selector, 1, 2, 2, 3,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_vpos_selector), 4);
	tx_vpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Top"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Middle"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Bottom"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_vpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (tx_vpos_selector), tx_vpos_selector_menu);

	max_vpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (max_vpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "max_vpos_selector", max_vpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (max_vpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), max_vpos_selector, 1, 2, 3, 4,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (max_vpos_selector), 4);
	max_vpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Top"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Middle"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Bottom"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_vpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (max_vpos_selector), max_vpos_selector_menu);

	device_vpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (device_vpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_vpos_selector", device_vpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_vpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), device_vpos_selector, 1, 2, 4, 5,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (device_vpos_selector), 4);
	device_vpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Top"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Middle"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_vpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Bottom"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_vpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (device_vpos_selector), device_vpos_selector_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (device_vpos_selector), 2);

	rx_hpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (rx_hpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_hpos_selector", rx_hpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_hpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), rx_hpos_selector, 2, 3, 1, 2,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_hpos_selector), 4);
	rx_hpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Left"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Center"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Right"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (rx_hpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (rx_hpos_selector), rx_hpos_selector_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (rx_hpos_selector), 1);

	tx_hpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (tx_hpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_hpos_selector", tx_hpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_hpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), tx_hpos_selector, 2, 3, 2, 3,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_hpos_selector), 4);
	tx_hpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Left"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Center"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Right"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (tx_hpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (tx_hpos_selector), tx_hpos_selector_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (tx_hpos_selector), 2);

	max_hpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (max_hpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "max_hpos_selector", max_hpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (max_hpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), max_hpos_selector, 2, 3, 3, 4,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (max_hpos_selector), 4);
	max_hpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Left"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Center"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Right"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (max_hpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (max_hpos_selector), max_hpos_selector_menu);

	device_hpos_selector = gtk_option_menu_new ();
	gtk_widget_ref (device_hpos_selector);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_hpos_selector", device_hpos_selector,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_hpos_selector);
	gtk_table_attach (GTK_TABLE (label_table), device_hpos_selector, 2, 3, 4, 5,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (device_hpos_selector), 4);
	device_hpos_selector_menu = gtk_menu_new ();
	glade_menuitem = gtk_menu_item_new_with_label (_("Left"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Center"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_hpos_selector_menu), glade_menuitem);
	glade_menuitem = gtk_menu_item_new_with_label (_("Right"));
	gtk_widget_show (glade_menuitem);
	gtk_menu_append (GTK_MENU (device_hpos_selector_menu), glade_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (device_hpos_selector), device_hpos_selector_menu);

	rx_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (rx_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_color_picker", rx_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_color_picker);
	gtk_table_attach (GTK_TABLE (label_table), rx_color_picker, 3, 4, 1, 2,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_color_picker), 4);

	tx_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (tx_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_color_picker", tx_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_color_picker);
	gtk_table_attach (GTK_TABLE (label_table), tx_color_picker, 3, 4, 2, 3,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_color_picker), 4);

	max_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (max_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "max_color_picker", max_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (max_color_picker);
	gtk_table_attach (GTK_TABLE (label_table), max_color_picker, 3, 4, 3, 4,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (max_color_picker), 4);

	device_color_picker = gnome_color_picker_new ();
	gtk_widget_ref (device_color_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_color_picker", device_color_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_color_picker);
	gtk_table_attach (GTK_TABLE (label_table), device_color_picker, 3, 4, 4, 5,
	                  (GtkAttachOptions) (0),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (device_color_picker), 4);

	rx_font_picker = gnome_font_picker_new ();
	gtk_widget_ref (rx_font_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "rx_font_picker", rx_font_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rx_font_picker);
	gtk_table_attach (GTK_TABLE (label_table), rx_font_picker, 4, 5, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (rx_font_picker), 4);
	gnome_font_picker_set_preview_text (GNOME_FONT_PICKER (rx_font_picker), "012345aAbBcC");

	tx_font_picker = gnome_font_picker_new ();
	gtk_widget_ref (tx_font_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "tx_font_picker", tx_font_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tx_font_picker);
	gtk_table_attach (GTK_TABLE (label_table), tx_font_picker, 4, 5, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (tx_font_picker), 4);
	gnome_font_picker_set_preview_text (GNOME_FONT_PICKER (tx_font_picker), "012345aAbBcC");

	max_font_picker = gnome_font_picker_new ();
	gtk_widget_ref (max_font_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "max_font_picker", max_font_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (max_font_picker);
	gtk_table_attach (GTK_TABLE (label_table), max_font_picker, 4, 5, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (max_font_picker), 4);
	gnome_font_picker_set_preview_text (GNOME_FONT_PICKER (max_font_picker), "012345aAbBcC");

	device_font_picker = gnome_font_picker_new ();
	gtk_widget_ref (device_font_picker);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "device_font_picker", device_font_picker,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (device_font_picker);
	gtk_table_attach (GTK_TABLE (label_table), device_font_picker, 4, 5, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (device_font_picker), 4);
	gnome_font_picker_set_preview_text (GNOME_FONT_PICKER (device_font_picker), "012345aAbBcC");

	item_label = gtk_label_new (_("Item"));
	gtk_widget_ref (item_label);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "item_label", item_label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (item_label);
	gtk_table_attach (GTK_TABLE (label_table), item_label, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (item_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (item_label), 0, 0.5);

	nlabel3 = gtk_label_new (_("Labels"));
	gtk_widget_ref (nlabel3);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "nlabel3", nlabel3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nlabel3);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), nlabel3);

	main_separator = gtk_hseparator_new ();
	gtk_widget_ref (main_separator);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "main_separator", main_separator,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_separator);
	gtk_box_pack_start (GTK_BOX (main_box), main_separator, FALSE, TRUE, 4);

	buttonbox = gtk_hbutton_box_new ();
	gtk_widget_ref (buttonbox);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "buttonbox", buttonbox,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (buttonbox);
	gtk_box_pack_start (GTK_BOX (main_box), buttonbox, FALSE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (buttonbox), 0);

	load_theme_button = gtk_button_new_with_label ("");
	load_theme_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (load_theme_button)->child),
	                                 _("_Load Theme..."));
	gtk_widget_add_accelerator (load_theme_button, "clicked", accel_group,
	                            load_theme_button_key, GDK_MOD1_MASK, 0);
	gtk_widget_ref (load_theme_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "load_theme_button", load_theme_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (load_theme_button);
	gtk_container_add (GTK_CONTAINER (buttonbox), load_theme_button);
	GTK_WIDGET_SET_FLAGS (load_theme_button, GTK_CAN_DEFAULT);

	save_theme_button = gtk_button_new_with_label ("");
	save_theme_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (save_theme_button)->child),
	                                 _("_Save theme..."));
	gtk_widget_add_accelerator (save_theme_button, "clicked", accel_group,
	                            save_theme_button_key, GDK_MOD1_MASK, 0);
	gtk_widget_ref (save_theme_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "save_theme_button", save_theme_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (save_theme_button);
	gtk_container_add (GTK_CONTAINER (buttonbox), save_theme_button);
	GTK_WIDGET_SET_FLAGS (save_theme_button, GTK_CAN_DEFAULT);

	apply_button = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
	gtk_widget_ref (apply_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "apply_button", apply_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (apply_button);
	gtk_container_add (GTK_CONTAINER (buttonbox), apply_button);
	GTK_WIDGET_SET_FLAGS (apply_button, GTK_CAN_DEFAULT);

	ok_button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
	gtk_widget_ref (ok_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "ok_button", ok_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ok_button);
	gtk_container_add (GTK_CONTAINER (buttonbox), ok_button);
	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

	close_button = gnome_stock_button (GNOME_STOCK_BUTTON_CLOSE);
	gtk_widget_ref (close_button);
	gtk_object_set_data_full (GTK_OBJECT (pref_window), "close_button", close_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (close_button);
	gtk_container_add (GTK_CONTAINER (buttonbox), close_button);
	GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (pref_window), "delete_event",
			    GTK_SIGNAL_FUNC (on_pref_window_delete_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (pref_window), "destroy_event",
			    GTK_SIGNAL_FUNC (on_pref_window_destroy_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (load_theme_button), "clicked",
			    GTK_SIGNAL_FUNC (on_load_theme_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (save_theme_button), "clicked",
			    GTK_SIGNAL_FUNC (on_save_theme_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (apply_button), "clicked",
			    GTK_SIGNAL_FUNC (on_apply_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			    GTK_SIGNAL_FUNC (on_ok_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			    GTK_SIGNAL_FUNC (on_close_button_clicked),
			    NULL);

	gtk_widget_grab_focus (load_theme_button);
	gtk_widget_grab_default (load_theme_button);
	gtk_object_set_data (GTK_OBJECT (pref_window), "tooltips", tooltips);

	gtk_window_add_accel_group (GTK_WINDOW (pref_window), accel_group);

	/*
	 * initialize all settings PUT THIS INTO SEPARATE FUNC?
	 */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fixed_width), display_scheme.fixed_width);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fixed_height), display_scheme.fixed_height);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (width_spinbutton), display_scheme.width);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (height_spinbutton), display_scheme.height);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (timeout_spinbutton), display_scheme.timeout);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (background_color_picker), display_scheme.background_color.red,
				    display_scheme.background_color.green, display_scheme.background_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (top_edge_color_picker), display_scheme.top_edge_color.red,
				    display_scheme.top_edge_color.green, display_scheme.top_edge_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (bottom_edge_color_picker), display_scheme.bottom_edge_color.red,
				    display_scheme.bottom_edge_color.green, display_scheme.bottom_edge_color.blue, 65535);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_rx_graph_button), display_scheme.show_rx_graph);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_tx_graph_button), display_scheme.show_tx_graph);
	if (display_scheme.graph_order == RX_ON_TOP) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rx_on_top_button), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tx_on_top_button), FALSE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tx_on_top_button), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rx_on_top_button), FALSE);
	}
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_rx_avg_load_button), display_scheme.show_rx_avg);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_tx_avg_load_button), display_scheme.show_tx_avg);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (rx_graph_color_picker), display_scheme.rx_graph_color.red,
				    display_scheme.rx_graph_color.green, display_scheme.rx_graph_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (tx_graph_color_picker), display_scheme.tx_graph_color.red,
				    display_scheme.tx_graph_color.green, display_scheme.tx_graph_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (rx_avg_load_color_picker), display_scheme.rx_avg_color.red,
				    display_scheme.rx_avg_color.green, display_scheme.rx_avg_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (tx_avg_load_color_picker), display_scheme.tx_avg_color.red,
				    display_scheme.tx_avg_color.green, display_scheme.tx_avg_color.blue, 65535);
	gtk_option_menu_set_history (GTK_OPTION_MENU (rx_graph_style_selector), display_scheme.rx_graph_style);
	gtk_option_menu_set_history (GTK_OPTION_MENU (tx_graph_style_selector), display_scheme.tx_graph_style);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rx_rate_button), display_scheme.show_rx_load);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tx_rate_button), display_scheme.show_tx_load);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (max_rate_button), display_scheme.show_max_load);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (device_name_button), display_scheme.show_dev);
	gtk_option_menu_set_history (GTK_OPTION_MENU (rx_vpos_selector), (display_scheme.rx_load_position & POS_VERT) >> 2);
	gtk_option_menu_set_history (GTK_OPTION_MENU (tx_vpos_selector), (display_scheme.tx_load_position & POS_VERT) >> 2);
	gtk_option_menu_set_history (GTK_OPTION_MENU (max_vpos_selector), (display_scheme.max_load_position & POS_VERT) >> 2);
	gtk_option_menu_set_history (GTK_OPTION_MENU (device_vpos_selector), (display_scheme.dev_position & POS_VERT) >> 2);
	gtk_option_menu_set_history (GTK_OPTION_MENU (rx_hpos_selector), display_scheme.rx_load_position & POS_HORIZ);
	gtk_option_menu_set_history (GTK_OPTION_MENU (tx_hpos_selector), display_scheme.tx_load_position & POS_HORIZ);
	gtk_option_menu_set_history (GTK_OPTION_MENU (max_hpos_selector), display_scheme.max_load_position & POS_HORIZ);
	gtk_option_menu_set_history (GTK_OPTION_MENU (device_hpos_selector), display_scheme.dev_position & POS_HORIZ);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (rx_color_picker), display_scheme.rx_load_color.red,
				    display_scheme.rx_load_color.green, display_scheme.rx_load_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (tx_color_picker), display_scheme.tx_load_color.red,
				    display_scheme.tx_load_color.green, display_scheme.tx_load_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (max_color_picker), display_scheme.max_load_color.red,
				    display_scheme.max_load_color.green, display_scheme.max_load_color.blue, 65535);
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (device_color_picker), display_scheme.dev_color.red,
				    display_scheme.dev_color.green, display_scheme.dev_color.blue, 65535);
	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (rx_font_picker), display_scheme.rx_load_font_name);
	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (tx_font_picker), display_scheme.tx_load_font_name);
	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (max_font_picker), display_scheme.max_load_font_name);
	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (device_font_picker), display_scheme.dev_font_name);

	return pref_window;
}
